﻿/* -LICENSE-START-
 ** Copyright (c) 2018 Blackmagic Design
 **
 ** Permission is hereby granted, free of charge, to any person or organization
 ** obtaining a copy of the software and accompanying documentation covered by
 ** this license (the "Software") to use, reproduce, display, distribute,
 ** execute, and transmit the Software, and to prepare derivative works of the
 ** Software, and to permit third-parties to whom the Software is furnished to
 ** do so, all subject to the following:
 **
 ** The copyright notices in the Software and this entire statement, including
 ** the above license grant, this restriction and the following disclaimer,
 ** must be included in all copies of the Software, in whole or in part, and
 ** all derivative works of the Software, unless such copies or derivative
 ** works are solely in the form of machine-executable object code generated by
 ** a source language processor.
 **
 ** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 ** IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 ** FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 ** SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 ** FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 ** ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 ** DEALINGS IN THE SOFTWARE.
 ** -LICENSE-END-
 */
using System;
using System.Collections.Generic;
using System.Linq;
using DeckLinkAPI;

namespace DeckLinkCapture
{
    class Program
    {
        [MTAThread]
        static void Main(string[] args)
        {
            var iterator = new DeckLinkIterator();

            // Get first device and create input callbacl interface 
            var deckLink = iterator.NextDeckLink.First();
            var deckLinkInput = new DeckLinkInputDevice(deckLink);

            deckLinkInput.VideoFrameArrivedHandler += VideoFrameArrived;
            deckLinkInput.InputFormatChangedHandler += DisplayModeChanged;

            Console.WriteLine("Starting Capture... press any key to stop.");
            deckLinkInput.StartCapture();
            Console.ReadKey();

            Console.WriteLine("Stopping Capture... press any key to exit.");
            deckLinkInput.StopCapture();
            Console.ReadKey();
        }

        static void VideoFrameArrived(object sender, DeckLinkVideoFrameArrivedEventArgs e)
        {

        }

        static void DisplayModeChanged(object sender, DeckLinkInputFormatChangedEventArgs e)
        {

        }
    }

    public class DeckLinkIterator
    {
        private IDeckLinkIterator iterator;

        public DeckLinkIterator()
        {
            iterator = new CDeckLinkIterator();
        }

        public IEnumerable<IDeckLink> NextDeckLink
        {
            get
            {
                while (true)
                {
                    IDeckLink returnDeckLink;
                    iterator.Next(out returnDeckLink);

                    if (returnDeckLink == null)
                        yield break;
                    else
                        yield return returnDeckLink;
                }
            }
        }
    }
}
